/*

 Modified for CCD tests Juha Niinikoski / 2002



code adaptation 7.4.2002
unsigned int output 8.4.2002
temperature output 9.4.2002
kbhit & clear serial input functions added 11.4.2002
binary printout & get2hex added 11.4.2002
leading zero logic for temp printout 12.4.2002
LCD output switch added 7.6.2002
negative temp display 3.7.2002
overrun error handling added to keypressed function 27.8.2002 

2 digit BCD clock display 03.10.2002
3 digit positive / negative printout for temperatures 04.10.2002
io_switch added to character input routines 04.10.2002
get2hex function corrected 07.08.2003

*/

#include	<pic.h>
#include	<conio.h>
#include	<stdio.h>
#include	"delay.h"


void serial_setup(unsigned char baud_div)
{
	/* relates crystal freq to baud rate - see above and PIC16F87x data sheet under 'USART async. modes'

	BRGH=1, Fosc=16MHz
	------------------
	Baud		SPBRG
	7812,5		124
        9600    	103
	19200		51
	38400   	25
	57600   	16
	115200  	8
	
	*/

	/*
	 * Comms setup:
	 */

	SPBRG=baud_div;
	BRGH=1;			//data rate for sending

	SYNC=0;			//asynchronous
	SPEN=1;			//enable serial port pins
	CREN=1;			//enable reception
	SREN=0;			//no effect
	TXIE=0;			//disable tx interrupts
	RCIE=0;			//disable rx interrupts
	TX9=0;			//8-bit transmission
	RX9=0;			//8-bit reception
	TXEN=0;			//reset transmitter
	TXEN=1;			//enable the transmitter

//	CTS = 0;		//enable host transmission
}

unsigned char dummy;



#define clear_usart_errors_inline	\
if (OERR)				\
{					\
	TXEN=0;				\
	TXEN=1;				\
	CREN=0;				\
	CREN=1;				\
}					\
if (FERR)				\
{					\
	dummy=RCREG;			\
	TXEN=0;				\
	TXEN=1;				\
}


//writes a character to the serial port
void putch(unsigned char c)
{
	switch (io_sw)
		{
		case UART:			// 16F877 internal UART
			while(!TXIF)		// set when register is empty
			{
			clear_usart_errors_inline;
			}
			TXREG=c;	
		break;

		case MAX1: case MAX2:
			MAX3100_send(c);	// MAX3100 Serial 1 or Serial 2
		break;

		case LCD1: case LCD2:
			lcdoutch(c);		// LCD panel 1 (upper) or 2 (lower)
		break;

		}
}


//gets a character from the serial port without timeout
unsigned char getch(void)
{
char tempc;

	switch (io_sw)
		{
		case UART:
		while(!RCIF)
			{
				clear_usart_errors_inline;
			}
		tempc = RCREG;
		break;

		case MAX1: case MAX2:
			tempc = MAX3100_receive();	// MAX3100 Serial 1 or Serial 2
		break;

		case LCD1: case LCD2:
			tempc = 0;		// input from LCD = 0
		break;
		}

	return tempc;
}

/* Convert character to HEX nibble, return = 128 if non hex nibble */

char cvt_hex(unsigned char ch)
{
	if (ch < '0')
		return(128);
	if (ch <= '9')
		return(ch - '0');

	if (ch < 'A')
		return (128);
	if (ch <= 'F')
		return(ch - 'A' + 10);

	if (ch < 'a')
		return (128);
	if (ch <= 'f')
		return(ch - 'a' + 10);
	return(128);
}


/* Gets byte (2 hex characters), no echo, no check if hex character */

unsigned char get2hex()
{
	unsigned char input, echo;

	echo = getch();
	input = cvt_hex(echo);
	echo = getch(); 
	echo = cvt_hex(echo);
	input = (input << 4) | echo;

	return input;
}



//check if character available, but do not read character serial port
//returns 0 if no characters available. Returns 1 if character available.
unsigned char keypressed(void)
{
	clear_usart_errors_inline;
	if(RCIF)
	return(1);
	else
	return(0);
}

//clear serial port receiver
void clear_usart_input(void)
{
	clear_usart_errors_inline;
	dummy = RCREG;
}


void clear_usart_errors(void)
{
	clear_usart_errors_inline;
}



/*
writes a character to the serial port in hex
*/

void putchhex(unsigned char c)
{
	unsigned char temp;

	// transmits in hex

	temp=c;

	c=(c >> 4);
	if (c<10) c+=48; else c+=55;
	putch(c);

	c=temp;

	c=(c & 0x0F);
	if (c<10) c+=48; else c+=55;
	putch(c);
}

void putinthex(unsigned int c)
{
	#define ramuint(x)				(*((unsigned int *) (x)))
	#define ramuint_hibyte(x)			(*(((unsigned char *)&x)+1))
	#define ramuint_lobyte(x)			(*(((unsigned char *)&x)+0))
	#define ramuchar(x)				(*((unsigned char *) (x)))

	putchhex(ramuint_hibyte(c));
	putchhex(ramuint_lobyte(c));

	#undef ramuint(x)
	#undef ramuint_hibyte(x)
	#undef ramuint_lobyte(x)
	#undef ramuchar(x)
}



/*
writes a character to the serial port in decimal
*/
void putchdec(unsigned char c)
{
	unsigned char temp;

	temp=c;
	//hundreds
	putch((c/100)+'0');
	c-=(c/100)*100;

	//tens
	putch((c/10)+'0');
	c-=(c/10)*10;

	//ones
	putch((c/1)+'0');
}

/*
Converts to BCD and writes out. Number must be between 0 and 99
*/
void put2dec( char c)
{
	if(c > 99)
		c = 99;

	//tens
	putch((c/10)+'0');
	c-=(c/10)*10;

	//ones
	putch((c/1)+'0');
}

/*
Converts to BCD and writes out. Number must be between -99 and +127
*/
void put3dec( char c)
{
	if(c & 0x80)		// test if negative
		{
		c = 0 - c;
		if(c > 99)
			c = 99;
		putch('-');
		}
	else
		{
		if ((c/100)>0) 
			putch((c/100)+'0');
		else
			putch(' ');
		}
	c-=(c/100)*100;

	//tens
	putch((c/10)+'0');
	c-=(c/10)*10;

	//ones
	putch((c/1)+'0');
}


/*
writes int to serial port in decimal
If DP = 0 no decimal point is printed.
DP !=0 = DP place from right.
This is used for Temp printout
simple negative number printout added 3.7.2002
minus sign is displayed at thousands digit place
*/
void putintdec( int c, char dp)
{
	int temp;
	unsigned char neg_sign;

	if (c > 9999)		/* clip to max reading */
		c = 9999;
	if (c < 0)		/* check if negative value */
		{
		neg_sign = 1;
		c = 0 - c;	/* invert it */
		if (c > 999)
			c = 999;	/* max negative reading */
		}
	else
		neg_sign = 0;
	temp=c;

	//thousands
	if (neg_sign == 1)
		putch('-');
	else
		{	
		if ((c/1000)>0 || (dp >= 3))
			putch((c/1000)+'0');
		else
			putch(' ');
		}
	c-=(c/1000)*1000;
	if (dp == 3)
		putch(',');

	//hundreds
	if ( ((temp/100)>0) || ((temp/1000)>0) || (dp >= 2))
		 putch((c/100)+'0');
	else
		putch(' ');	
	c-=(c/100)*100;

	if (dp == 2)
		putch(',');

	//tens
	if ( ((temp/10)>0) || ((temp/100)>0) || ((temp/1000)>0) || (dp >= 1) )
 		putch((c/10)+'0');
	else
		putch(' ');
	c-=(c/10)*10;

	if (dp == 1)
		putch(',');

	//ones
	putch((c/1)+'0');

}

/*
writes a character to the serial port in binary
*/
void putchbin(unsigned char ch)
{
	unsigned char x;

	for(x=0; x < 8; x++)
	{
	if (ch & 0x80)
		putch('1');
	else
		putch('0');
	ch = ch << 1;
	}

}

void putst(register const char *str)
{
	while((*str)!=0)
	{
		putch(*str);
    if (*str==13) putch(10);
    if (*str==10) putch(13);
		str++;
	}
}

