/*

 CCD Display board LCD routines
 This is polling driver with 8-bit interface

 Juha Niinikoski 01.10.2002

*/

/* 4 * 20 LCD display memory layout */
#define LINE_1     0x00         /* line 1 start address */
#define LINE_2     0x40         /* line 2 start address */
#define LINE_3     0x14         /* line 3 start address */
#define LINE_4     0x54         /* line 4 start address */

#define CURSET     0x80         /* set memory cursor */
#define	CLEAR	   0x01		/* clear display */


void	clk_lcd( unsigned char ldata )		// set data & clock both lcd controllers
	{
	LCD_DATA = ldata;
	if(io_sw == LCD1)			// select display
		LCD_E1 = 1;
//	else
//		LCD_E2 = 1;
	asm("nop");
	LCD_E1 = 0;
//	LCD_E2 = 0;
	}


void	wait_lcd_rdy(void)			// wait until lcd module is ready
	{
	unsigned char BF_AC;			// BF flag & address counter

	LEDA = 1;
	BF_AC = 0x80;				// set busy for first round
	LCD_TRIS = 0xFF;			// make data port input
	LCD_RW = 1;				// read
	LCD_RS = 0;				// command mode
	while(BF_AC & 0x80)	
		{
		if(io_sw == LCD1)		// select display
			LCD_E1 = 1;
//		else
//			LCD_E2 = 1;
		asm("nop");
		asm("nop");
		BF_AC = LCD_DATA;
		LCD_E1 = 0;
//		LCD_E2 = 0;
		}
	LCD_RW = 0;
	LCD_RS = 1;
	LCD_TRIS = 0x00;			// return to write mode

	LEDA = 0;
	}

void set_cur_lcd( unsigned char cur)		// set cursor position at selected display
	{
	wait_lcd_rdy( );
	LCD_RS = 0;				// switch command mode		
	clk_lcd(cur | CURSET);			// set cursor command + cursor position
	LCD_RS = 1;				// back to data mode
	}

void clear_lcd(void )				// clear selected display
	{
	wait_lcd_rdy( );
	LCD_RS = 0;				// switch command mode		
	clk_lcd(CLEAR);				// clear display. Display going to be busy 1,6 mS
	LCD_RS = 1;				// back to data mode
	}


void	initlcd(void)		// init selected LCD controller
	{
	LCD_RS = 0;		// set command mode
	LCD_RW = 0;
	clk_lcd(0x38);		// write init data with delays
	delay(5);
	clk_lcd(0x38);
	delay(1);
	clk_lcd(0x38);		//8-bit, 2 rows, 5x7 matrix, display on
	delay(1);		//cursor off, no blink
	clk_lcd(0x0C);
	delay(1);
	clk_lcd(0x01);
	delay(2);	
	clk_lcd(0x06);		// increment, no display shift
	delay(1);
	LCD_RS = 1;		// set data mode
	}


void	lcdoutch( unsigned char lcd_char )	// output character to selected display
	{
		wait_lcd_rdy();
		clk_lcd(lcd_char);	
	}

