/*

 CCD Display board MAX3100 uart driver

 Juha Niinikoski 01.10.2002

*/

//**********************
// 16F877 SPI functions
//**********************

// Init SPI controller

void initspi(void)
{
	CS_SERIAL1 = 1;	/* disable MAX3100 chips */

	SSPEN = 0;	/* disable SPI if it was enabled */
	SSPCON = 0x00;	/* SPI master, Idle low CKP = 0, Fosc/4 */
	SSPSTAT = 0x40;	/* clear status register, SMP = 0, CKE = 1 */
	SSPEN = 1;
	SSPIF = 0;	/* clear irq flag */
}

// Transmit and Receive SPI

unsigned char txrxspi (unsigned char txc)
{

	SSPBUF = txc;		/* start transmission */
	while (!STAT_BF)
	  ;			/* Wait until ready */

	if (WCOL || SSPOV)
		{
		WCOL = 0;
		SSPOV = 0;
		}

return (SSPBUF);
}


//**********************
// MAX 3100 primitives
//**********************


#define MODEC RIE         /* mode cbyte: receive int enable */
#define MODED BR384       /* mode dbyte: 19200 bit/s */

#define SPIRATE (0 << SPR1) | (0 << SPR0)            /* SPI rate: fast */
#define SPIMODE (1 << SPE) | (1 << MSTR) | SPIRATE   /* SPI mode */



//   Write to MAX3100 UART

static void uart_write(unsigned char cbyte, unsigned char dbyte)
	{
	if(io_sw == MAX1)	// select MAX3100	
		CS_SERIAL1 = 0;
//	else
//		CS_SERIAL2 = 0;
	txrxspi(cbyte);
	txrxspi(dbyte);
	CS_SERIAL1 = 1;
//	CS_SERIAL2 = 1;
	}	
	

//   Read from MAX3100 UART

static void uart_read(unsigned char cbyte, unsigned char *resph, unsigned char *respl)
	{
	if(io_sw == MAX1)	// select MAX3100	
		CS_SERIAL1 = 0;
//	else
//		CS_SERIAL2 = 0;
	*resph = txrxspi(cbyte);
	*respl = txrxspi(0);
	CS_SERIAL1 = 1;
//	CS_SERIAL2 = 1;
	}


//   Get MAX3100 UART status

static unsigned char uart_status(void)
	{
	unsigned char cfgc, cfgd;
	
	uart_read(RDCFG, &cfgc, &cfgd);
	return cfgc;
	}

//   Check for receiver ready

unsigned char MAX3100_rxrdy(void)
	{
	return (uart_status() & RXRDY) != 0;
	}


//   Check for transmitter ready

unsigned char MAX3100_txrdy(void)
	{
	return (uart_status() & TXRDY) != 0;
	}



//   Receive a character

unsigned char MAX3100_receive(void)
	{
	unsigned char sts, ch;

	while (!MAX3100_rxrdy())
		;

	uart_read(RDDAT, &sts, &ch);
	return ch;
	}


//   Send a character

void MAX3100_send(unsigned char ch)
	{
	while (!MAX3100_txrdy())
		;

	uart_write(WRDAT | RTS, ch);
	}

//   Initialize MAX3100 serial I/O

void MAX3100_init(void)
	{

	initspi();	/* init 16F877 SPI interface */

	uart_write(WRCFG | (MODEC), MODED); /* set UART mode */
	
	while(MAX3100_rxrdy())    /* flush UART receiver */
		MAX3100_receive();
	}


