/*

Magnet IO Board Tester

 Juha Niinikoski 24.4.2002

 Modified for CCD display board MAX3100 UARTs 30.09.2002

*/

//**********************
// 16F877 SPI functions
//**********************

// Init SPI controller

void initspi(void)
{
	CS_SERIAL1 = 1;	/* disable MAX3100 chips */
	CS_SERIAL2 = 1;

	SSPEN = 0;	/* disable SPI if it was enabled */
	SSPCON = 0x02;	/* SPI master, Idle low, Fosc/64 */
	SSPEN = 1;
	SSPSTAT = 0x00;	/* clear status register */
	SSPIF = 0;	/* clear irq flag */
}

// Transmit and Receive SPI

unsigned char txrxspi (unsigned char txc)
{

	SSPBUF = txc;		/* start transmission */
	while (!STAT_BF)
	  ;			/* Wait until ready */

	if (WCOL || SSPOV)
		{
		WCOL = 0;
		SSPOV = 0;
		}

return (SSPBUF);
}


//**********************
// MAX 3100 primitives
//**********************


#define MODEC RIE         /* mode cbyte: receive int enable */
#define MODED BR96        /* mode dbyte: 9600 bit/s */

#define SPIRATE (0 << SPR1) | (0 << SPR0)            /* SPI rate: fast */
#define SPIMODE (1 << SPE) | (1 << MSTR) | SPIRATE   /* SPI mode */



//   Write to MAX3100 UART

static void uart_write(unsigned char cbyte, unsigned char dbyte)
	{	
	CS_SERIAL1 = 0;
	txrxspi(cbyte);
	txrxspi(dbyte);
	CS_SERIAL1 = 1;
	}	
	

//   Read from MAX3100 UART

static void uart_read(unsigned char cbyte, unsigned char *resph, unsigned char *respl)
	{
	CS_SERIAL1 = 0;
	*resph = xch_spi(cbyte);
	*respl = xch_spi(0);
	CS_SERIAL1 = 1;
	}


//   Get MAX3100 UART status

static unsigned char uart_status(void)
	{
	unsigned char cfgc, cfgd;
	
	uart_read(RDCFG, &cfgc, &cfgd);
	return cfgc;
	}

//   Check for receiver ready

unsigned char siox_rxrdy(void)
	{
	return (uart_status() & RXRDY) != 0;
	}


//   Check for transmitter ready

unsigned char siox_txrdy(void)
	{
	return (uart_status() & TXRDY) != 0;
	}



//   Receive a character

unsigned char siox_receive(void)
	{
	BYTE sts, ch;

	while (!siox_rxrdy())
		;

	uart_read(RDDAT, &sts, &ch);
	return ch;
	}


//   Send a character

void siox_send(BYTE ch)
	{
	while (!siox_txrdy())
		;

	uart_write(WRDAT | RTS, ch);
	}

//   Initialize MAX3100 serial I/O

void MAX3100_init(void)
	{

	initspi();	/* init 16F877 SPI interface */

	uart_write(WRCFG | (MODEC), MODED); /* set UART mode */
	
	while(siox_rxrdy())    /* flush UART receiver */
		siox_receive();
	}


